﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap.FlatData;
using Infragistics.Olap;
using System.Threading;
using System.Collections.ObjectModel;
using System.Diagnostics;

namespace XamPivotGridFlatData
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly Random _rand;

        public MainWindow()
        {
            _rand = new Random();
            InitializeComponent();
            Loaded += OnLoaded;
        }

        public void OnLoaded(object sender, RoutedEventArgs args)
        {
            var data = GetData();
            var flatDataSource = new FlatDataSource
            {
                Cube = DataSourceBase.GenerateInitialCube("cube"),
                ItemsSource = data,
            };
            DataContext = flatDataSource;
            PeriodicallyChangeData(data, 4);
        }

        private void PeriodicallyChangeData(IList<MyPivotItemType> data, int numberToUpdatePerIteration)
        {
            new Thread(() =>
            {
                while (true)
                {
                    Dispatcher.Invoke((Action)(() =>
                    {
                        for (int i = 0; i < numberToUpdatePerIteration; i++)
                        {
                            int n = _rand.Next() % data.Count;
                            var item = data[n];
                            item.Quantity += (decimal)_rand.NextDouble() - .5m;
                            data.RemoveAt(n);
                            data.Insert(data.Count-2, item) ;
                        }
                    }));
                    Thread.Sleep(1000);
                }
            }).Start();
        }

        private IList<MyPivotItemType> GetData()
        {
            var data = (
                from broker in new[] { "Al", "Abe", "Alan", "Ariel", "Adrian", "Anthony", "Anabelle", "Abdullah", "Alexander" }
                from account in new[] { "Bo", "Ben", "Brad", "Bella", "Barney", "Bernard", "Beatrice", "Broderick", "Barrington" }
                from asset in new[] { "lead", "feathers", "dirt", "platinum" }
                select new MyPivotItemType
                {
                    Broker = broker,
                    Asset = asset,
                    Account = account,
                    Quantity = (decimal)_rand.NextDouble() * 100 - 50
                }
            ).ToList();
            return new ObservableCollection<MyPivotItemType>(data);
        }
    }
    
    internal class MyPivotItemType
    {
        public string Asset { get; set; }
        public string Broker { get; set; }
        public string Account { get; set; }
        public decimal Quantity { get; set; }
    }
}


